local addonName, L = ...;

local TIME_LIMIT, BROADCAST, BROADCAST_CHAT_TYPE;

local function OnUpdate(self, elapsed)

	if ( not self.configMode ) then
		if ( self.timeButton.active ) then
			self.elapsed = self.elapsed + elapsed;
			if ( self.elapsed >= 1 ) then
				self.elapsed = 0;
				
				local currTime = self.timeButton.time + 1;
				
				if ( not self.timeButton.finished and currTime < TIME_LIMIT ) then
					self.timeButton.time = currTime;
					
					local minutes = math.floor(self.timeButton.time / 60);
					local seconds = self.timeButton.time - ( 60 * minutes );
					
					if ( string.len(minutes) == 1 ) then
						minutes = "0"..minutes;
					end
					
					if ( string.len(seconds) == 1 ) then
						seconds = "0"..seconds;
					end
					
					self.timeButton.text:SetText(string.format(L["SCOREBOARD_TIME_FRAME_TEXT"], minutes, seconds));
				else
					
					local minutes = math.floor(TIME_LIMIT / 60);
					local seconds = TIME_LIMIT - ( 60 * minutes );
									
					if ( string.len(minutes) == 1 ) then
						minutes = "0"..minutes;
					end
									
					if ( string.len(seconds) == 1 ) then
						seconds = "0"..seconds;
					end

					self.timeButton.text:SetText(string.format(L["SCOREBOARD_TIME_FRAME_TEXT"], minutes, seconds));						
					
					if ( BROADCAST ) then
						-- Inform Players that the time is over:
						SendChatMessage(L["SCOREBOARD_TIME_LIMIT_REACHED"] ,BROADCAST_CHAT_TYPE);
					end					
					
					ALSpec_ScoreBoard:StopTimer()
				end
			end
		end
		
		if ( self.timeButton.finished ) then
			self.elapsed = self.elapsed + elapsed;
			if ( self.elapsed >= 0.5 ) then
				self.elapsed = 0;
				if ( self.timeButton:GetAlpha() == 0 ) then
					self.timeButton:SetAlpha(1);
				else
					self.timeButton:SetAlpha(0);
				end
			end
		end
	else
		if ( self.timeButton.active ) then
			ALSpec_ScoreBoard:StopTimer();
			ALSpec_ScoreBoard:ResetTimer();
		end
		self.timeButton:SetAlpha(1);
	end

end

local function TimeButton_OnClick(self, button ,down)

	if ( button == "LeftButton" ) then
		if ( ALSpec_ScoreBoard.configMode ) then
			InterfaceOptionsFrame_Show();
			InterfaceOptionsFrame_OpenToCategory(ALSpec_Options);
		elseif ( self.finished ) then
			ALSpec_ScoreBoard:ResetTimer();
		elseif ( self.active ) then
			
			ALSpec_ScoreBoard:StopTimer();
			
			if ( BROADCAST ) then
				-- Inform players that the timer was canceled:
				SendChatMessage(L["SCOREBOARD_TIME_CANCELED"] ,BROADCAST_CHAT_TYPE);
			end
			ALSpec_ScoreBoard:ResetTimer();
		elseif ( ALSpec_CountDown.count ) then
			if ( BROADCAST ) then
				-- Inform players that the Countdown was canceled:
				SendChatMessage(L["COUNTDOWN_BROADCAST_CANCELED"] ,BROADCAST_CHAT_TYPE);
			end			
			ALSpec_CountDown:Reset();
		else
			ALSpec_CountDown:Start();
		end
	elseif ( button == "RightButton" ) then
		if ( ALSpec_CountDown.count ) then
			if ( BROADCAST ) then
				-- Inform players that the Countdown was canceled:
				SendChatMessage(L["COUNTDOWN_BROADCAST_CANCELED"] ,BROADCAST_CHAT_TYPE);
			end				
			
			ALSpec_CountDown:Reset()
		end
		ArenaLiveSpectator:Toggle()
	end

end

local function TeamButton_OnClick(self, button, down)

	if ( button == "LeftButton" ) then
		if ( ALSpec_ScoreBoard.configMode ) then
			ALSpec_TeamMenu:Open(self.side);
		else
			ALSpec_ScoreBoard:RaiseScore(self.side);
		end
	elseif ( button == "RightButton" ) then
		if ( ALSpec_ScoreBoard.configMode ) then
			ALSpec_ScoreBoard:ResetScore(self.side)
		else
			ALSpec_ScoreBoard:ReduceScore(self.side)
		end	
	end

end

local function ScoreBoardButton_OnEnter(button)
	
	if ( ALSpec_ScoreBoard.configMode ) then
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(button);

		if ( button.tooltipTitle ) then
			GameTooltip:AddLine(button.tooltipTitle, 1 , 0.82, 0);
		end
		
		if ( button.tooltip ) then
			GameTooltip:AddLine(button.tooltip, 1 , 1, 1);
		end
		
		GameTooltip:Show();
	end
	
	
end

local function ScoreBoardButton_OnLeave(button)
	GameTooltip:ClearLines()
	GameTooltip:Hide()
end

function ALSpec_ScoreBoard:Initialise()
	
	local prefix = self:GetName();
	local leftScore = ArenaLiveCore:GetDBEntry(addonName, "Teams/Left/Score");
	local rightScore = ArenaLiveCore:GetDBEntry(addonName, "Teams/Right/Score");
	
	self.elapsed = 0;
	
	self.leftTeam = _G[prefix.."LeftTeam"];
	self.leftTeam.side = "Left";
	self.leftTeam.tooltip = L["SCOREBOARD_TEAMNAME_TOOLTIP"];
	self.leftTeam.tooltipTitle = L["SCOREBOARD_TEAMNAME_TOOLTIP_TITLE"];
	self.leftTeam:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self.rightTeam = _G[prefix.."RightTeam"];
	self.rightTeam.side = "Right";
	self.rightTeam.tooltip = L["SCOREBOARD_TEAMNAME_TOOLTIP"];
	self.rightTeam.tooltipTitle = L["SCOREBOARD_TEAMNAME_TOOLTIP_TITLE"];
	self.rightTeam:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self.timeButton = _G[prefix.."TimeButton"];
	self.timeButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self.timeButton.tooltip = L["SCOREBOARD_TIMEFRAME_TOOLTIP"];
	self.timeButton.tooltipTitle = L["SCOREBOARD_TIMEFRAME_TOOLTIP_TITLE"];	
	-- Start with config mode, since UIParent is visible at first.
	ALSpec_ScoreBoard:SetMode(true);	
	
	-- Set texts (some text objects were defined in the .xml-file via parentKey):
	self.leftTeam.score:SetText(leftScore);
	self.rightTeam.score:SetText(rightScore);
	ALSpec_ScoreBoard:UpdateTeamName("Left");
	ALSpec_ScoreBoard:UpdateTeamName("Right");
	
	-- Set Team Icons:
	ALSpec_ScoreBoard:UpdateTeamIcon("Left");
	ALSpec_ScoreBoard:UpdateTeamIcon("Right");
	
	-- Set Timer:
	ALSpec_ScoreBoard:UpdateTimeLimit()
	
	-- Set Tournament Icon:
	ALSpec_ScoreBoard:UpdateTournamentIcon()
	
	-- Set Scripts:
	self.leftTeam:SetScript("OnClick", TeamButton_OnClick);
	self.leftTeam:SetScript("OnEnter", ScoreBoardButton_OnEnter);
	self.leftTeam:SetScript("OnLeave", ScoreBoardButton_OnLeave);
	self.rightTeam:SetScript("OnClick", TeamButton_OnClick);
	self.rightTeam:SetScript("OnEnter", ScoreBoardButton_OnEnter);
	self.rightTeam:SetScript("OnLeave", ScoreBoardButton_OnLeave);
	self.timeButton:SetScript("OnClick", TimeButton_OnClick);
	self.timeButton:SetScript("OnEnter", ScoreBoardButton_OnEnter);
	self.timeButton:SetScript("OnLeave", ScoreBoardButton_OnLeave);
	
	ArenaLiveOptions:InitialiseButton(ALSpec_ScoreBoardShowStatisticBoardButton, addonName, nil, function(self) if ( ALSpec_MatchStatistic:IsShown() ) then ALSpec_MatchStatistic:Hide() else ALSpec_MatchStatistic:Show(); end end, L["STATISTIC_TOGGLE_BUTTON_TEXT"]);
end

function ALSpec_ScoreBoard:SetMode(configMode)

	if ( configMode ) then
		self.timeButton.text:SetText(L["SCOREBOARD_CONFIG_MODE_TEXT"]);
		self.configMode = true;
	else
		self.timeButton.text:SetText(string.format(L["SCOREBOARD_TIME_FRAME_TEXT"], "00", "00"));
		self.configMode = nil;
	end

end

function ALSpec_ScoreBoard:UpdateTeamName(side)

	local frame;
	if ( side == "Left" ) then
		frame = self.leftTeam;
	elseif ( side == "Right" ) then
		frame = self.rightTeam;
	end
	
	if ( not frame ) then
		return;
	end
	
	local teamName = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..side.."/Name");
	local teamTag = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..side.."/Tag");
	local maxWidth = frame.name:GetWidth();
	
	self.fontSizeCheck:SetText(teamName);
	local textWidth = self.fontSizeCheck:GetWidth();
	
	if ( textWidth > maxWidth ) then
	
		if ( teamTag and teamTag ~= "" ) then
			frame.name:SetText(teamTag);
		else
			frame.name:SetText(teamName);
		end
	else
		frame.name:SetText(teamName);
	end

end

function ALSpec_ScoreBoard:UpdateTeamIcon(side)

	local frame;
	if ( side == "Left" ) then
		frame = self.leftTeam;
	elseif ( side == "Right" ) then
		frame = self.rightTeam;
	end
	
	if ( not frame ) then
		return;
	end
	
	local teamIcon = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..side.."/Icon");
	
	frame.icon:SetTexture(teamIcon);
end

function ALSpec_ScoreBoard:RaiseScore(side)
	
	local text;
	if ( side == "Left" ) then
		text = self.leftTeam.score;
	elseif ( side == "Right" ) then
		text = self.rightTeam.score;
	end
	
	if ( not text ) then
		return;
	end
	
	local dbKey = "Teams/"..side.."/Score";	
	local score = ArenaLiveCore:GetDBEntry(addonName, dbKey);
	
	if ( score ) then
		score = score + 1;
		ArenaLiveCore:SetDBEntry(addonName, dbKey, score);
		text:SetText(score);
	end

end

function ALSpec_ScoreBoard:ReduceScore(side)
	
	local text;
	if ( side == "Left" ) then
		text = self.leftTeam.score;
	elseif ( side == "Right" ) then
		text = self.rightTeam.score;
	end
	
	if ( not text ) then
		return;
	end
	
	local dbKey = "Teams/"..side.."/Score";	
	local score = ArenaLiveCore:GetDBEntry(addonName, dbKey);
	
	if ( score ) then
		score = score - 1;
		if ( score >= 0 ) then
			ArenaLiveCore:SetDBEntry(addonName, dbKey, score);
			text:SetText(score);
		end
	end

end

function ALSpec_ScoreBoard:ResetScore(side)
	
	local text;
	if ( side == "Left" ) then
		text = self.leftTeam.score;
	elseif ( side == "Right" ) then
		text = self.rightTeam.score;
	end
	
	if ( not text ) then
		return;
	end
	local dbKey = "Teams/"..side.."/Score";	
	ArenaLiveCore:SetDBEntry(addonName, dbKey, 0);
	text:SetText(0);
end

function ALSpec_ScoreBoard:StartTimer()
	self.timeButton.active = true;
	self.timeButton.finished = nil;
	self.timeButton.time = 0;
	self.elapsed = 0;
	self:SetScript("OnUpdate", OnUpdate);
end

function ALSpec_ScoreBoard:StopTimer()

	self.timeButton.active = false;
	self.timeButton.finished = true;
	
	-- Let main frame know that the match is over:
	ArenaLiveSpectator:OnEvent("ALSPEC_MATCH_FINISHED");

end

function ALSpec_ScoreBoard:ResetTimer()
	self:SetScript("OnUpdate", nil);
	self.timeButton.active = nil;
	self.timeButton.finished = nil;
	self.elapsed = 0;
	
	self.timeButton:SetAlpha(1);
	
	if ( self.configMode ) then
		ALSpec_ScoreBoard:SetMode(true);
	else
		ALSpec_ScoreBoard:SetMode(false);
	end
	
end

function ALSpec_ScoreBoard:UpdateTimeLimit()

	local minutes = ArenaLiveCore:GetDBEntry(addonName, "TimeLimit/Minutes");
	local seconds = ArenaLiveCore:GetDBEntry(addonName, "TimeLimit/Seconds");
	
	TIME_LIMIT = ( minutes * 60 ) + seconds;
	BROADCAST = ArenaLiveCore:GetDBEntry(addonName, "Countdown/Broadcast");
	BROADCAST_CHAT_TYPE = ArenaLiveCore:GetDBEntry(addonName, "Countdown/BroadcastChannel");
end

function ALSpec_ScoreBoard:UpdateTournamentIcon()
	
	local tournamentIcon = ArenaLiveCore:GetDBEntry(addonName, "Scoreboard/TournamentIcon");
	
	self.tournamentIcon:SetTexture(tournamentIcon);

end