local addonName, L = ...;

-- Load TeamCache Handler:
local TeamCache = ArenaLiveCore:GetHandler("TeamCache");
local UnitTracker = ArenaLiveCore:GetHandler("UnitTracker");

-- Create Variables to store the maximal number of players of a particular unitID group.
local MAX_PARTY_MEMBERS = 4;
local MAX_RAID_MEMBERS = 40;
local MAX_ARENA_ENEMIES = 5;

ALSPEC_AVAILABLE_PLAYERS = {};	-- Table for TeamMember dropdown options.

local function MemberDropDown_OnClick(dropdownButton, unit)

	local dropdown = UIDROPDOWNMENU_OPEN_MENU;
	local id = dropdown:GetID();
	
	-- Update Text; We can simply use .value here since it is the same as text anyways.:
	if ( dropdownButton.value == "" ) then
		UIDropDownMenu_SetText(dropdown, L["TEAM_MENU_MEMBER_DROPDOWN_EMPTY_TEXT"]);
	elseif ( dropdownButton.value == "ENTERNAME" ) then
		StaticPopup_Show("ARENALIVESPECTATOR_ENTER_PLAYER_NAME");
	else
		UIDropDownMenu_SetText(dropdown, dropdownButton.value);
	end
	
	if ( dropdownButton.value ~= "ENTERNAME" ) then
		ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Member"..id, dropdownButton.value);
		UnitTracker:UpdateUnit(unit);
		ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_MEMBER_UPDATE", ALSpec_TeamMenu.currentTeam, id);
	end
end

local info = {};
local function MemberDropDown_Refresh(dropdown)
	
	if ( not dropdown:IsVisible() ) then
		return;
	end

	local id = dropdown:GetID();
	
	ALSpec_TeamMenu:RefreshAvailablePlayers();
	local memberName = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Member"..id);
	
	
	-- First the option to reset the player:
	info.text = L["TEAM_MENU_MEMBER_DROPDOWN_RESET"];
	info.value = "";
	info.func = MemberDropDown_OnClick;
	info.checked = nil;
	info.colorCode = nil;
	UIDropDownMenu_AddButton(info);
	
	-- Then the option to add a name:
	info.text = L["TEAM_MENU_MEMBER_DROPDOWN_ENTER_NAME"];
	info.value = "ENTERNAME";
	info.func = MemberDropDown_OnClick;
	info.checked = nil;
	info.colorCode = nil;
	UIDropDownMenu_AddButton(info);	
	
	for playerName, unit in pairs(ALSPEC_AVAILABLE_PLAYERS) do
		local _, class = UnitClass(unit);
		local colourCode;

		if ( class ) then
			colourCode = "|c"..(RAID_CLASS_COLORS[class]["colorStr"]);
		end
		
		info.text = playerName;
		info.value = playerName;
		info.func = MemberDropDown_OnClick;
		info.arg1 = unit;
		info.colorCode = colourCode;
		
		if ( memberName == info.value ) then
			UIDropDownMenu_SetText(dropdown, info.text);
			info.checked = true;
		else
			info.checked = nil;
		end
		
		UIDropDownMenu_AddButton(info);
	end
end

local function MemberDropDown_Initialise(dropdown)

	local prefix = dropdown:GetName();
	local id = dropdown:GetID();
	
	local title = _G[prefix.."Title"];
	
	-- Create reference in parent frame:
	ALSpec_TeamMenu["memberDropDown"..id] = dropdown;
	
	-- Set dropdown title
	title:SetText(string.format(L["TEAM_MENU_MEMBER_DROPDOWN_TITLE"], id));
	
	-- Initialise the dropdown using blizzard's standard functionset.
	UIDropDownMenu_Initialize(dropdown, MemberDropDown_Refresh);
end

local function TeamEditBox_OnFocusLost(self)
	local value = self:GetText();
	ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/"..self.key, value);
	
	local event;
	if ( self.key == "Name" ) then
		event = "ALSPEC_TEAM_NAME_UPDATE";
	elseif ( self.key == "Tag" ) then
		event = "ALSPEC_TEAM_TAG_UPDATE";
	elseif ( self.key == "Icon" ) then
		event = "ALSPEC_TEAM_ICON_UPDATE";
	end
	
	ArenaLiveSpectator:OnEvent(event, ALSpec_TeamMenu.currentTeam);
end

local function TeamEditBox_Initialise(editBox, savedVariableKey, titleLocalisation)
	local prefix = editBox:GetName();
	local title = _G[prefix.."Title"];
	
	-- set key:
	editBox.key = savedVariableKey;
	
	-- Set Title
	title:SetText(titleLocalisation);
	
	editBox:SetScript("OnEditFocusLost", TeamEditBox_OnFocusLost);
end

local function SaveTeamButton_OnClick(self, button, down)

	if ( button == "LeftButton" ) then		
		TeamCache:SetTeam(ALSpec_TeamMenu.currentTeam);
	end

end

local function SaveTeamButton_Initialise(button)
	local prefix = button:GetName();
	local text = _G[prefix.."Text"];
	-- Set Text
	button:SetText(L["TEAM_MENU_SAVE_TEAM_BUTTON"]);

	-- Get new text width to set the size of the button correctly:
	if ( text ) then
		local width = text:GetWidth() + 20;
		button:SetWidth(width);
	end
	
	button:SetScript("OnClick", SaveTeamButton_OnClick);
end

local function TeamCacheDropDown_OnClick(button)

	local dropdown = UIDROPDOWNMENU_OPEN_MENU;

	if ( ALSpec_TeamMenu.currentTeam ) then
		-- Update Text; We can simply use .value here since it is the same as text anyways:
		UIDropDownMenu_SetText(dropdown, button.value);
		
		local name, tag, icon, member1, member2, member3 = TeamCache:GetTeam(button.value);
		
		if ( name ) then
		
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Name", name);
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Tag", tag);
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Icon", icon);
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Member1", member1);
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Member2", member2);
			ArenaLiveCore:SetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Member3", member3);
			
			-- Update Team Menu:
			ALSpec_TeamMenu:OnShow();
			ArenaLiveSpectator:CheckAllSideFrameUnits();
			
			-- Let the addon know that the team has changed:
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_NAME_UPDATE", ALSpec_TeamMenu.currentTeam);
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_TAG_UPDATE", ALSpec_TeamMenu.currentTeam);
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_ICON_UPDATE", ALSpec_TeamMenu.currentTeam);
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_MEMBER_UPDATE", ALSpec_TeamMenu.currentTeam, 1);
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_MEMBER_UPDATE", ALSpec_TeamMenu.currentTeam, 2);
			ArenaLiveSpectator:OnEvent("ALSPEC_TEAM_MEMBER_UPDATE", ALSpec_TeamMenu.currentTeam, 3);
		end
		
	end

end

function ALSpec_TeamMenu:Initialise()
	
	-- Reference for title fontstring:
	self.title = ALSpec_TeamMenuTitle;
	
	-- Initialise Team Cache Dropdown:
	TeamCache:InitialiseDropdown(ALSpec_TeamMenuTeamCacheDropDown, TeamCacheDropDown_OnClick, L["TEAM_CACHE_DROPDOWN_TITLE"]);
	
	-- Initialise TextBoxes:
	TeamEditBox_Initialise(ALSpec_TeamMenuNameEditBox, "Name", L["TEAM_MENU_TEAM_NAME_TITLE"])
	TeamEditBox_Initialise(ALSpec_TeamMenuTagEditBox, "Tag", L["TEAM_MENU_TEAM_TAG_TITLE"])
	TeamEditBox_Initialise(ALSpec_TeamMenuIconEditBox, "Icon", L["TEAM_MENU_TEAM_ICON_TITLE"])
	
	-- Initialise Team Member Dropdowns:
	MemberDropDown_Initialise(ALSpec_TeamMenuMember1DropDown);
	MemberDropDown_Initialise(ALSpec_TeamMenuMember2DropDown);
	MemberDropDown_Initialise(ALSpec_TeamMenuMember3DropDown);
	
	-- Initialise Buttons:
	SaveTeamButton_Initialise(ALSpec_TeamMenuSaveTeamButton);
	ALSpec_TeamMenuCloseButtonText:SetText(L["CLOSE"]);
	ALSpec_TeamMenuCloseButton:SetScript("OnClick", function(self, button, down) if ( button == "LeftButton" ) then ALSpec_TeamMenu:Close(); end end);
	self:SetScript("OnShow", ALSpec_TeamMenu.OnShow);
end

function ALSpec_TeamMenu.OnShow(self)
	
	if ( not self.currentTeam ) then
		ArenaLiveCore:Message(L["TEAM_MENU_ERROR_NO_CURRENT_TEAM"], "error");
		self:Hide();
		return;
	end
	
	local gameMode = ArenaLiveCore:GetDBEntry(addonName, "GameMode");
	
	-- Set Title text:
	self.title:SetText(string.format(L["TEAM_MENU_TITLE"], L[self.currentTeam]));
	
	-- Set Team Cache DropDown Text:
	UIDropDownMenu_SetText(ALSpec_TeamMenuTeamCacheDropDown, L["TEAM_CACHE_DROPDOWN_DEFAULT_TEXT"]);
	
	-- Set Team Name:
	local teamName = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Name") or "";
	ALSpec_TeamMenuNameEditBox:SetText(teamName);
	
	-- Set Team Tag:
	local teamTag = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Tag") or "";
	ALSpec_TeamMenuTagEditBox:SetText(teamTag);
	
	-- Set Team Icon:
	local teamIcon = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..ALSpec_TeamMenu.currentTeam.."/Icon") or "";
	ALSpec_TeamMenuIconEditBox:SetText(teamIcon);
	
	-- Show/Hide Dropdowns and set their text according to saved variables:
	for i = 1, 3 do
		local dropdown = ALSpec_TeamMenu["memberDropDown"..i];
		
		if ( i <= gameMode ) then
			local memberName = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..self.currentTeam.."/Member"..i);
			
			if ( memberName and memberName ~= "" ) then
				UIDropDownMenu_SetText(dropdown, memberName);
			else
				UIDropDownMenu_SetText(dropdown, L["TEAM_MENU_MEMBER_DROPDOWN_EMPTY_TEXT"]);
			end
			dropdown:Show();
		else
			dropdown:Hide();
		end
	end
end

-- Team: "Left" or "Right".
function ALSpec_TeamMenu:Open(team)
	
	-- Need OnShow script.
	if ( self:IsShown() and team ~= self.currentTeam ) then
		self:Hide();
	end
	
	self.currentTeam = team;
	self:Show();
end

function ALSpec_TeamMenu:Close()
	self:Hide();
end

function ALSpec_TeamMenu:RefreshAvailablePlayers()

	local unit, name;
	
	-- Reset the table:
	table.wipe(ALSPEC_AVAILABLE_PLAYERS);
	
	-- First collect the player:
	name = ArenaLiveSpectator:GetUnitName("player");
	if ( name ) then
		ALSPEC_AVAILABLE_PLAYERS[name] = "player";
	end
	
	-- Next party/raid:
	if ( IsInRaid() ) then
		for i = 1, MAX_RAID_MEMBERS do	
			unit = "raid"..i;
			if ( UnitExists(unit) ) then
				name = ArenaLiveSpectator:GetUnitName(unit);

				if ( name and not ALSPEC_AVAILABLE_PLAYERS[name] ) then
					ALSPEC_AVAILABLE_PLAYERS[name] = unit;
				end
			else
				break;
			end
		end
	elseif ( IsInGroup() ) then
		for i = 1, MAX_PARTY_MEMBERS do
			unit = "party"..i;
			if ( UnitExists(unit) ) then
				name = ArenaLiveSpectator:GetUnitName(unit);
			
				if ( name and not ALSPEC_AVAILABLE_PLAYERS[name] ) then
					ALSPEC_AVAILABLE_PLAYERS[name] = unit;
				end	
			else
				break;
			end
		end
	end
	
	-- Arena Targets:
	for i = 1, MAX_ARENA_ENEMIES do
		unit = "arena"..i;
		name = ArenaLiveSpectator:GetUnitName(unit);
		
		if ( name and not ALSPEC_AVAILABLE_PLAYERS[name] ) then
			ALSPEC_AVAILABLE_PLAYERS[name] = unit;
		end			
	end
	
	-- Last the Target:
	name = ArenaLiveSpectator:GetUnitName("target");
	if ( UnitIsPlayer("target") and name and not ALSPEC_AVAILABLE_PLAYERS[name] ) then
		ALSPEC_AVAILABLE_PLAYERS[name] = "target";
	end	
end